/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision mediump float;

uniform lowp sampler2D s_TexUnit0;
uniform lowp sampler2D s_TexUnit1;
uniform lowp sampler2D s_TexUnit2;
uniform lowp sampler2D s_TexUnit3;

uniform lowp vec3 u_sLightDirection;
uniform lowp vec3 u_sLightDiffuse;
uniform vec2 u_sLightConeParams;

uniform	lowp vec3 u_dLightAmbient;
uniform	lowp vec3 u_dLightDiffuse;
uniform	vec3 u_dLightDirection;
uniform	vec3 u_dLightHalfDirection;

varying mediump vec2 v_TexCoord0; // field texture
varying mediump vec2 v_TexCoord1; // weight map
varying mediump vec2 v_TexCoord2; // windrow texture
#if !defined(NORMAL_MAPPING)
varying lowp vec3 v_Color;
varying lowp vec3 v_Normal;
#endif
varying mediump vec3 v_sLightDir;

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
#endif

const lowp vec3 cultivatedColor = vec3(0.49, 0.38, 0.27);
const lowp vec3 acreColor = vec3(0.51, 0.39, 0.28);

void main()
{
    lowp vec2 tex0 = texture2D(s_TexUnit0, v_TexCoord0).rg; // Field Ground texture - only RG are used
    lowp vec4 weight = texture2D(s_TexUnit1, v_TexCoord1); // Weight Texture, (R, G, B, A) made on the fly
    lowp vec4 windrowTexture = texture2D(s_TexUnit2, v_TexCoord2);
    lowp vec3 groundColor = mix(acreColor * tex0.g, cultivatedColor * tex0.r, weight.r) + vec3(weight.g);
    lowp float windrowAlpha = windrowTexture.a*weight.a;
    lowp vec3 finalTextureColor = mix(groundColor, windrowTexture.xyz, windrowAlpha);

#if defined(NORMAL_MAPPING)
    mediump vec4 tex = texture2D(s_TexUnit3, v_TexCoord0);
    lowp vec4 normalCombined = normalize(tex*2.0 - 1.0);
    
    // Properly reconstructing the third normal vector component (to get a vector of length 1).
    //lowp vec3 normalCultivated = vec3(normalCombined.rg, 1.0);
    lowp vec3 normalCultivated = vec3(normalCombined.rg, sqrt(1.0 - dot(normalCombined.rg, normalCombined.rg)));
    
    // Properly reconstructing the third normal vector component (to get a vector of length 1).
    //lowp vec3 normalAcre = vec3(normalCombined.ba, 1.0);
    lowp vec3 normalAcre = vec3(normalCombined.ba, sqrt(1.0 - dot(normalCombined.ba, normalCombined.ba)));
    //lowp vec3 normal = normalize(mix(normalAcre, normalCultivated, weight.r) * vec3(0.1, 0.1, 1.0));
    lowp vec3 normal = mix(normalAcre, normalCultivated, weight.r) * vec3(0.1, 0.1, 1.0);
    
    lowp mat3 TSToLS;
    TSToLS[0] = v_TSToLS1;
    TSToLS[1] = v_TSToLS2;
    TSToLS[2] = v_TSToLS3;
    
    //normal = TSToLS * normal;
    normal = normalize(TSToLS * normal);
#else
    lowp vec3 normal = v_Normal;
#endif

    lowp vec3 lighting = vec3(0,0,0);
#if defined(VERTEX_LIGHTING)
    lighting += v_Color;
#else
    lighting += u_dLightAmbient;
#endif
    
    // spot light diffuse
    float attn = clamp(1.0 - dot(v_sLightDir, v_sLightDir), 0.0, 1.0);
    attn *= attn;
    mediump vec3 sLightDir = normalize(v_sLightDir);
    float spotCosAngle = dot(sLightDir, u_sLightDirection);
    float spotFactor = clamp(spotCosAngle*u_sLightConeParams.x + u_sLightConeParams.y, 0.0, 1.0);
    float spotNdotL = clamp(dot(normal, sLightDir)*5.0, 0.0, 1.0);
    lowp vec3 spotPower = vec3(spotNdotL*spotFactor * attn);
    lighting += spotPower*u_sLightDiffuse;
    
    // directional light diffuse + specular
    float dLightScale = 0.0;
#if !defined(VERTEX_LIGHTING)
    dLightScale += max(dot(u_dLightDirection, normal), 0.0);
#endif
    lowp vec3 dLightScaleVec = vec3(dLightScale);
    lighting += dLightScaleVec * u_dLightDiffuse;

    gl_FragColor = vec4(finalTextureColor * lighting, 1.0);
}
