/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision highp float;
precision mediump int;

#if defined(ENV_MAPPING)
#define LIGHT_SPACE_WORLD_SPACE
#endif

#if !defined(NORMAL_MAPPING)
#define VERTEX_LIGHTING
#endif

uniform mat4 u_mwMatrix;
uniform mat4 u_mvMatrix;
uniform mat4 u_mvpMatrix;
uniform mediump vec3 u_dLightAmbient;
uniform mediump vec3 u_dLightDiffuse;
#if defined(LIGHT_SPACE_WORLD_SPACE)
uniform	mediump vec4 u_sLightPositionWS;
uniform mediump vec3 u_dLightDirectionWS;
#define u_sLightPosition u_sLightPositionWS
#define u_dLightDirection u_dLightDirectionWS
#else
uniform	mediump vec4 u_sLightPosition;
uniform mediump vec3 u_dLightDirection;
#endif

// for env mapping
uniform	vec3 u_cameraPosWS;


#if defined(UV_SCROLL) || defined(UV_SCROLL_ANIM)
uniform vec4 u_uvOffset;
#endif

attribute vec3 Position;
#if !defined(TIRE_TRACK)
attribute vec4 Normal;
#endif
#if defined(NORMAL_MAPPING) || defined(TIRE_TRACK)
attribute vec4 Tangent;
#endif
attribute vec2 TexCoord0;


// Varyings

varying mediump vec2 v_TexCoord0;
#if !defined(NORMAL_MAPPING)
varying mediump vec3 v_Normal;
#endif
varying mediump vec3 v_sLightDir;

#if defined(VERTEX_LIGHTING)
varying mediump vec3 v_Color;
#endif

#if defined(ENV_MAPPING)
#if defined(NORMAL_MAPPING)
varying mediump vec3 v_ViewDir;
#else
varying mediump vec3 v_Reflection;
#endif
#endif

#if defined(NORMAL_MAPPING)
// transforms from light space (world or view) to tangent space
varying mediump vec3 v_TSToLS1;
varying mediump vec3 v_TSToLS2;
varying mediump vec3 v_TSToLS3;
// doesn't work on tegra 2:
//varying mediump mat3 v_TSToLS;
#endif

#if defined(TIRE_TRACK) || defined(PARTICLE_SYSTEM)
varying mediump float v_TTPSData_a;
#endif
#if defined(PARTICLE_SYSTEM)
varying mediump float v_TTPSData_x;
#endif

#if defined(LIGHT_SPACE_WORLD_SPACE)
#define u_MSToLSMatrix u_mwMatrix
#else
#define u_MSToLSMatrix u_mvMatrix
#endif

#if defined(UV_SCROLL_ANIM)
varying mediump vec4 v_attribute0;
#endif

#if defined(DISTANCE_FADING)
varying highp vec4 v_screenPos;
#endif

void main()
{
    v_TexCoord0 = TexCoord0;
#if defined(UV_SCROLL) || defined(UV_SCROLL_ANIM)
    v_TexCoord0 += u_uvOffset.xy;
#endif
    vec4 pos=vec4(Position,1.0);
	
#if defined(UV_SCROLL_ANIM)
    vec4 worldPos = u_mwMatrix * pos;
    v_attribute0 = vec4(0, 0, 0, worldPos.y);
#endif    

    vec4 projPosition = u_mvpMatrix * pos;	
    gl_Position = projPosition;
#if defined(DISTANCE_FADING)
    v_screenPos.xy = projPosition.xy*0.5 + 0.5*projPosition.w;
    v_screenPos.z = projPosition.z;
    v_screenPos.w = projPosition.w;
#endif
 
#if !defined(TIRE_TRACK)
    vec3 normalMS = Normal.xyz;
#else
    vec3 normalMS = vec3(0.0, 1.0, 0.0);
#endif

    vec3 posLS = (u_MSToLSMatrix * pos).xyz;
    vec3 normalLS = normalize((u_MSToLSMatrix * vec4(normalMS,0.0)).xyz);
#if !defined(NORMAL_MAPPING)
	v_Normal = normalLS;
#endif
    v_sLightDir = (u_sLightPosition.xyz - posLS) / 50.0;
    
#if defined(NORMAL_MAPPING)
    // use transpose of matrix by swapping multiplication (GLSL addresses rows, not columns)
    v_TSToLS1 = normalize((u_MSToLSMatrix * vec4(Tangent.xyz,0)).xyz);
#if defined(TIRE_TRACK)
    v_TSToLS2 = normalize((u_MSToLSMatrix * vec4(cross(normalMS, Tangent.xyz)*1.0,0)).xyz);
#else
    v_TSToLS2 = normalize((u_MSToLSMatrix * vec4(cross(normalMS, Tangent.xyz)*Tangent.w,0)).xyz);
#endif
    v_TSToLS3 = normalize((u_MSToLSMatrix * vec4(normalMS,0)).xyz);
#endif

#if defined(VERTEX_LIGHTING)
    // directional light diffuse + ambient
    float ndotl = max(dot(normalLS, u_dLightDirection), 0.0);
    v_Color = (ndotl*u_dLightDiffuse) + u_dLightAmbient;
#endif

    // calculate viewing direction in world space (note: light space is world space if we do env mapping)
#if defined(ENV_MAPPING)
    vec3 viewDir = normalize(u_cameraPosWS - posLS);
#if !defined(NORMAL_MAPPING)
    // calc reflected angle for environment map
    v_Reflection=-reflect(viewDir,normalLS);
#else
    v_ViewDir = viewDir;
#endif
#endif

#if defined(TIRE_TRACK)
    v_TTPSData_a = Tangent.a;
#endif
#if defined(PARTICLE_SYSTEM)
    v_TTPSData_a = Normal.a;
    v_TTPSData_x = pos.y*5.0;
#endif


#ifdef SWITCH_XY
    // switch coords to landscape mode for iphone
    gl_Position.xy=vec2(gl_Position.y, -gl_Position.x);
#endif
}
