/*
 * Copyright (c) 2008-2012 GIANTS Software GmbH, Confidential, All Rights Reserved.
 * Copyright (c) 2003-2012 Christian Ammann and Stefan Geiger, Confidential, All Rights Reserved.
 */

precision highp float;
precision mediump int;

uniform mat4 u_mwMatrix;
uniform mat4 u_mvpMatrix;
uniform mediump vec4 u_sLightPositionWS;
uniform mediump vec3 u_dLightDirectionWS;
#define u_sLightPosition u_sLightPositionWS
#define u_dLightDirection u_dLightDirectionWS
uniform vec3 u_cameraPosWS;
uniform vec4 u_uvOffset;

attribute vec3 Position;
attribute vec4 Normal;
attribute vec4 Tangent;

varying highp vec2 v_TexCoord0;
#ifndef VIVANTE_VARYING_BUGFIX
// The Vivante GC1000 has a maximum of 8 varying vectors as returned by glGetIntegerv(GL_MAX_VARYING_VECTORS).
varying highp vec2 v_TexCoord1;
varying highp vec2 v_TexCoord2;
#endif

varying mediump vec3 v_sLightDir;
varying lowp vec3 v_ViewDir;
varying lowp vec3 v_TSToLS1;
varying lowp vec3 v_TSToLS2;
varying lowp vec3 v_TSToLS3;
varying highp vec4 v_attribute0;
varying mediump vec3 v_sLightHalfDir;

#define u_MSToLSMatrix u_mwMatrix

void main()
{
    vec4 pos = vec4(Position, 1.0);
    gl_Position = u_mvpMatrix * pos;
    vec3 posLS = (u_MSToLSMatrix * pos).xyz;
    v_sLightDir = (u_sLightPosition.xyz - posLS) / 50.0;
    
    float textureScale = u_uvOffset.z;
    vec2 bumpSpeed = u_uvOffset.xy * textureScale;
    vec4 worldPos = u_mwMatrix * pos;
    vec2 texCoords = worldPos.xz * textureScale;
    vec2 translation = vec2(bumpSpeed.x, 0.0);
    vec2 translation2 = bumpSpeed;
    
    v_TexCoord0 = texCoords.xy + translation;
#ifndef VIVANTE_VARYING_BUGFIX    
    v_TexCoord1 = texCoords.xy * 2.0 + translation * 4.0;
    v_TexCoord2 = texCoords.xy * 4.0 + translation2;
#endif

    v_attribute0.xy = texCoords.xy + translation2 * 0.75;
    v_attribute0.w = u_uvOffset.w;
    
    v_sLightHalfDir = normalize(u_cameraPosWS -worldPos.xyz) + u_dLightDirectionWS;

    // use transpose of matrix by swapping multiplication (GLSL addresses rows, not columns)
    v_TSToLS1 = normalize((u_MSToLSMatrix * vec4(Tangent.xyz,0)).xyz);
    v_TSToLS2 = normalize((u_MSToLSMatrix * vec4(cross(Normal.xyz, Tangent.xyz)*Tangent.w,0)).xyz);
    v_TSToLS3 = normalize((u_MSToLSMatrix * vec4(Normal.xyz,0)).xyz);
    v_ViewDir = normalize(u_cameraPosWS - posLS);

#ifdef SWITCH_XY
    gl_Position.xy=vec2(gl_Position.y, -gl_Position.x);
#endif
}
